// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockaddress.h 942 2011-11-27 18:25:59Z fancylibs $
**
** Copyright (C) 2011 ITEG IT-Engineers GmbH
**
** Author: Wolfgang Glas
**
** Classes for handling socket addresses.
**
*****************************************************************************/

#ifndef FANCYSOCK_ADDRESS_H
#define FANCYSOCK_ADDRESS_H

#include <fancysock.h>
#include <fancyobject.h>

#ifdef WIN32
#define AF_LOCAL 99
extern "C" {
	struct sockaddr_un
	{
		short sun_family;
		char  sun_path[108];
	};
}
#endif

extern "C" {
  struct sockaddr;
}

namespace Fancy {
  namespace Sock {

  /**
   * An abstract base class, which encapsulates the polymorphic
   * POSIX \c struct \c sockaddr.
   */
  class FANCYSOCK_API Address : virtual public Fancy::Object
  {
  protected:
	  Address() {}
	  virtual ~Address();

  public:
	  typedef Fancy::ObjPtr<Address> ptr_t;

	  /**
	   *
	   * Pure virtual function, which returns the pointer to the \c sockaddr
	   * structure of any derived class for usage in any POSIX socket function.
	   */
	  virtual const struct sockaddr *sockAddr() const = 0;

	  /**
	   * Pure virtual function, which returns the size of the \c sockaddr
	   * structure of any derived class for usage in any POSIX socket function.
	   */
	  virtual uint32_t sockLen() const = 0;

	  /**
	   * Return a string, which contains no port number for IP addresses.
	   * For local socket addresses, this function returns the same
	   * result than toString().
	   */
	  virtual std::string getHostString() const = 0;

	  /**
	   * Pure virtual function, which returns the domain of a socket corresponding
	   * to an address, as required for the first argument to socket(2).
	   */
	  virtual int addrDomain() const = 0;

	  static ptr_t newInstance(const struct sockaddr *addr, uint32_t len);

	  static ptr_t newInstance (const std::string &address);

	  static ptr_t newLocalInstance (const std::string &path);

	  static ptr_t newIP4Instance (uint32_t addr, int port);

	  static ptr_t resolve (const std::string &name, int def_port=0);

	  static std::string getLocalPrefix();
  };

  }
}

#endif // ! FANCYSOCK_ADDRESS_H
