/*
 * fancysockbeaninvocationhandlerfactory.h
 *
 *  Created on: 09.08.2009
 *      Author: wglas
 */

#ifndef FANCYSOCKBEANINVOCATIONHANDLERFACTORY_H_
#define FANCYSOCKBEANINVOCATIONHANDLERFACTORY_H_

#include <fancysock.h>
#include <fancysockcallbackinterfaceaware.h>

namespace Fancy {

class UID;

namespace Sock {

class InvocationHandler;
class InvocationAuthenticator;
class RemotingConnectionHandlerFactory;
class SessionManager;

class FANCYSOCK_API BeanInvocationHandlerFactory: public virtual CallbackInterfaceAware {
protected:
    BeanInvocationHandlerFactory();
    virtual ~BeanInvocationHandlerFactory();
public:
    typedef Fancy::ObjPtr<BeanInvocationHandlerFactory> ptr_t;

    /**
     * Return a new instance.
     */
    static ptr_t newInstance();

    /**
     * Introduce an authenticator, which is called for each call.
     */
    virtual void setInvocationAuthenticator(const Fancy::ObjPtr<InvocationAuthenticator> &ia) = 0;

    /**
     * @return the authenticator used for each call.
     */
    virtual Fancy::ObjPtr<InvocationAuthenticator> getInvocationAuthenticator() const = 0;

    /**
     * @param manager The session manager which maintains a sever-side session.
     */
    virtual void setSessionManager(const Fancy::ObjPtr<SessionManager> &manager) = 0;

    /**
     * @return The session manager which maintains a sever-side session.
     */
    virtual Fancy::ObjPtr<SessionManager> getSessionManager() const = 0;

    /**
     * @return The handler, which handles calls to returned callback proxies.
     */
    virtual Fancy::ObjPtr<InvocationHandler> newReturnCallbackHandler() = 0;

    /**
     * @return an invocation handler, which handles open and close session requests.
     */
    virtual Fancy::ObjPtr<InvocationHandler> newSessionContextHandler() = 0;

    /**
     * Generate an invocation handler, which covers the given reflected interface.
     *
     * @param target The reflect target as returned be the static @c reflect method of a remote interface.
     */
    virtual Fancy::ObjPtr<InvocationHandler> newInvocationHandler(const Fancy::ObjPtr<Fancy::ReflectionTarget> &target) = 0;

    /**
     * Generate an invocation handler, which covers the given bean.
     *
     * @param Bean The type of the bean to generate an invocation handler for.
     */
    template <class Bean>
    Fancy::ObjPtr<InvocationHandler> newBeanInvocationHandler(const Fancy::ObjPtr<Bean> &bean) {
        return this->newInvocationHandler(Bean::reflect(bean));
    }
};

}

}

#endif /* FANCYSOCKBEANINVOCATIONHANDLERFACTORY_H_ */
