// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockemailtransport.h 768 2010-08-17 08:46:00Z fancylibs $
**
** Copyright (C) 2005 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A basic email client implementation for transporting arbitrary documents.
**
*****************************************************************************/

#ifndef FANCYSOCK_EMAIL_TRANSPORT_H
#define FANCYSOCK_EMAIL_TRANSPORT_H

#include <fancysocktransport.h>

namespace Fancy {
  namespace Sock {
    class Address;

    class FANCYSOCK_API EmailClientTransport : virtual public Transport
    {
    protected:
      EmailClientTransport();
      virtual ~EmailClientTransport();

    public:
      typedef Fancy::ObjPtr<EmailClientTransport> ptr_t;

      enum AuthenticationMechanism { None, Plain, CramMD5 };

      static ptr_t newInstance();

      virtual void setHelo (const std::string &h) = 0;
      virtual void setSender (const std::string &s) = 0;
      virtual void setAddressee (const std::string &addr) = 0;
      virtual void setRelay (const Fancy::ObjPtr<Address> &relay) = 0;
      virtual void setTimeout(int msec) = 0;

      /**
       * Sets the authentication information for SMTP AUTH.
       *
       * Some mechanisms may not be implemented or not be supported by the server, so expect Exceptions.
       */
      virtual void setAuthentication(AuthenticationMechanism mechanism, const std::string &user, const std::string &password) = 0;

      virtual const std::string &getSender() const = 0;
      virtual const std::string &getAddressee() const = 0;
      virtual const std::string &getHelo() const = 0;
      virtual Fancy::ObjPtr<Address> getRelay() const = 0;
      virtual int                getTimeout() const = 0;

    private:
      EmailClientTransport (const EmailClientTransport &);
      EmailClientTransport &operator=(const EmailClientTransport &);
    };
  }
}

#endif // ! FANCYSOCK_EMAIL_TRANSPORT_H
