/*
 * fancysockinvocationhandler.h
 *
 *  Created on: 26.07.2009
 *      Author: wglas
 */

#ifndef FANCYSOCKINVOCATIONHANDLER_H_
#define FANCYSOCKINVOCATIONHANDLER_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class Map;
template <class T> class ObjectArray;

namespace Sock {

class InvocationContext;

/**
 * An invocation handler, which is registered under a subsystem name at a server
 * or handles callbacks on a client.
 */
class FANCYSOCK_API InvocationHandler: public virtual Fancy::Object {
protected:
	InvocationHandler();
	virtual ~InvocationHandler();
public:
	typedef Fancy::ObjPtr<InvocationHandler> ptr_t;

	/**
 	 * @param ctx The context under which the call has been issued.
	 * @param arg The argument of the call.
	 * @param metadata The metadata accompanying the remote call, which is passed to the
	 *                 invocation handler. Metadata may be used to communicate authorization
	 *                 information such as user/password or a session Id.
	 * @return The result of the operation.
	 */
	virtual Object::ptr_t invoke(const Fancy::ObjPtr<InvocationContext> &ctx,
			                     const Fancy::ObjPtr<Fancy::Object> &arg,
			                     const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;
};

}

}

#endif /* FANCYSOCKINVOCATIONHANDLER_H_ */
