// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockpacketdevice.h 938 2011-11-25 22:12:25Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libxml.
** Extensible for using SSL or socks etc. by using many virtual functions.
**
*****************************************************************************/

#ifndef FANCYSOCK_PACKET_DEVICE_H
#define FANCYSOCK_PACKET_DEVICE_H

#include <fancypollablechannel.h>
#include <fancysockaddress.h>

namespace Fancy {
namespace Sock {

/** A base class for packet sockets.

    This class defines an socket device, which is represented by
    a POSIX filedecriptor as returned by socket(2).

    It may be reimplemented in order to use many other socket encapsulations
    such as SSL or SOCKS.
  */
  class FANCYSOCK_API PacketDevice : virtual public Fancy::PollableChannel
  {
  public:
	  typedef Fancy::ObjPtr<PacketDevice> ptr_t;

	  /**
	   * Instantiate a new ICMP instance.
	   */
	  static ptr_t newICMPInstance();

	  /**
	   * Instantiate a new UDP instance for client use.
	   */
	  static ptr_t newUDPInstance();

	  /**
	   * Instantiate and bind a new UDP instance and bind it to the given address.
	   */
	  static ptr_t bindUDP (const Address::ptr_t &addr);

	  /**
	   * Receive a datagram and return the number of bytes received.
	   * The addr pointer is set to the origin of the datagram.
	   */
	  virtual int recvFrom(void *buf, int len, Address::ptr_t &addr) = 0;

	  /**
	   * Send a datagram to the given address.
	   */
	  virtual int sendTo(const void *buf, int len, const Address::ptr_t &addr) = 0;

  protected:
	  PacketDevice();
	  virtual ~PacketDevice();
  };


  }
}

#endif // ! FANCYSOCK_PACKET_DEVICE_H
