/*
 * fancysockremotingclient.h
 *
 *  Created on: 30.07.2009
 *      Author: wglas
 */

#ifndef FANCYSOCKREMOTINGCLIENTPOOL_H_
#define FANCYSOCKREMOTINGCLIENTPOOL_H_

#include <fancysockinvocationcontext.h>

namespace Fancy {

class SerializableFactory;
class String;
class ThreadPool;

namespace Sock {

class InvocationHandler;
class OneWayHandler;
class DeviceFactory;
class RemotingClient;

class FANCYSOCK_API RemotingClientPool: public virtual Object {
protected:
	RemotingClientPool();
	virtual ~RemotingClientPool();

public:
	typedef Fancy::ObjPtr<RemotingClientPool> ptr_t;

	static ptr_t newInstance(size_t minSize, size_t maxSize,
	                         unsigned idleTimeout, unsigned waitTimeout,
	                         const std::string &url,
			                 const Fancy::ObjPtr<DeviceFactory> &deviceFactory,
			                 const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory);

	static ptr_t newInstance(size_t minSize, size_t maxSize,
                             unsigned idleTimeout, unsigned waitTimeout,
                             const std::string &url,
			                 const Fancy::ObjPtr<DeviceFactory> &deviceFactory,
			                 const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory,
			                 const Fancy::ObjPtr<Fancy::ThreadPool> &threadPool);

	/** must be called before start() */
	virtual void registerBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem,
			                              const Fancy::ObjPtr<OneWayHandler> &handler) = 0;

	virtual void deregisterBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) = 0;

	virtual Fancy::ObjPtr<OneWayHandler> getBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) const = 0;

    virtual int getBufferSize() const = 0;
    virtual void setBufferSize(int n) = 0;

    virtual Fancy::ObjPtr<RemotingClient> takeClient() = 0;

    virtual void putBackClient(const Fancy::ObjPtr<RemotingClient> &client) = 0;

    virtual void start() = 0;
    virtual void stop() = 0;
};

}

}

#endif /* FANCYSOCKREMOTINGCLIENTPOOL_H_ */
