/*
 * fancysocksessionmanager.h
 *
 *  Created on: 17.08.2009
 *      Author: wglas
 */

#ifndef FANCYSOCKSESSIONMANAGER_H_
#define FANCYSOCKSESSIONMANAGER_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class Map;
class UID;

namespace Sock {

/**
 * A session manager is used to maintain a server-side, per-client session.
 * The session information may be attached to the current thread or stored
 * in a per-client map, whatever suits best to your rquirements.
 *
 * Most likely, your @c SessionManager implementation will play together with
 * an @c InvocationAuthenticator implementation. It also possible to implement
 * both interfaces, @c SessionManager and @c InvocationAuthenticator in one
 * bean.
 */
class FANCYSOCK_API SessionManager: public virtual Fancy::Object {
protected:
    SessionManager();
    virtual ~SessionManager();
public:
    typedef Fancy::ObjPtr<SessionManager> ptr_t;

    /**
     * Open a session and attach it to the current thread.
     *
     * @param clientUID The client UID for which to open a session.
     * @param metadata The request metadata as passed to
     *                 @c RemotingBeanImport::setMetadata().
     */
    virtual void openSession(const Fancy::ObjPtr<Fancy::UID> &clietUID,
                             const Fancy::ObjPtr<Fancy::Map> metadata) = 0;

    /**
     * Close a session and withdraw it from the current thread.
     *
     * @param sessionId The client UID of which to close a session previously
     *                  created by @c openSession.
     */
    virtual void closeSession(const Fancy::ObjPtr<Fancy::UID> &clientUID) = 0;
};

}

}

#endif /* FANCYSOCKSESSIONMANAGER_H_ */
