// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysocktransport.h 686 2010-07-27 09:02:22Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps a document transport layer on top of a device.
**
*****************************************************************************/

#ifndef FANCYSOCK_TRANSPORT_H
#define FANCYSOCK_TRANSPORT_H

#include <string>
#include <fancysock.h>
#include <fancychannel.h>

namespace Fancy {

  class Map;
  class String;

  namespace Sock {

    class Device;

    /**
       An abstract base class, which encapsulates a transport
       protocol for transporting packets of a certain MIME type
       over a socket.
    */
    class FANCYSOCK_API Transport : virtual public Fancy::Channel
    {
    protected:
      Transport();

      virtual ~Transport();

    public:
      typedef Fancy::ObjPtr<Transport> ptr_t;

      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE;
      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE_PLAIN;
      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE_PLAIN_UTF8;
      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE_XML;
      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE_HTML;
      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE_HTML_UTF8;
      static const Fancy::ObjPtr<Fancy::String> CONTENT_TYPE_XHTML;

      /**
        Constructs a raw Transport instance using the given device and the
        given buffer size. A raw transport works by appending a zero byte
        after each chunk of data. Hence a raw transport is only suitable for
        @c text/plain or @c text/xml documents.
      */
      static ptr_t newRawInstance(const Fancy::ObjPtr<Device> &d,
    		                      int bufsize = 16384);

      /**
       * Open a document for writing. The document will be closed, when
       * @c flush() is called on this instance.
       *
       * @param headers Header fields to be written. The keys of this map are
       *                @c Fancy::String instances.
       */
      virtual void openWriteDoc(const Fancy::ObjPtr<Map> &headers);

      /**
       * Open a document for reading. Return the parsed headers fields.
       *
       * @param timeout A timeout in milliseconds to wait for the start of a document
       *                to arrive. A value of zero signifies to wait infinitely,
       *                which is not recommended in real-world networking applications.
       *                If nothing is sent by the peer within the given timeout a
       *                @c Fancy::Sock::CExeption with @c errno set to EAGAIN is thrown.
       */
      virtual Fancy::ObjPtr<Map> openReadDoc(int timeout) = 0;

    private:
      Transport (const Transport &);
      Transport &operator=(const Transport &);
    };

  }
}

#endif // ! FANCYSOCK_TRANSPORT_H
